#include <iostream>
#include <vector>

using namespace std;

struct point{
  int r;
  int c;
};


void Relax(const vector<vector<int> >& Grid, vector<vector<int> >& Distances, const vector<vector<bool> >& Visited, point p){
  if(p.r >0 && !Visited[p.r-1][p.c]){	       
    Distances[p.r-1][p.c] = min(Distances[p.r-1][p.c], Distances[p.r][p.c] + Grid[p.r-1][p.c]);
  }
  if(p.r < Grid.size()-1 && !Visited[p.r+1][p.c]){
    Distances[p.r+1][p.c] = min(Distances[p.r+1][p.c], Distances[p.r][p.c] + Grid[p.r+1][p.c]);
  }
  if(p.c >0 && !Visited[p.r][p.c-1]){
    Distances[p.r][p.c-1] = min(Distances[p.r][p.c-1], Distances[p.r][p.c] + Grid[p.r][p.c-1]);
  }
  if(p.c < Grid[0].size()-1 && !Visited[p.r][p.c+1]){
    Distances[p.r][p.c+1] = min(Distances[p.r][p.c+1], Distances[p.r][p.c] + Grid[p.r][p.c+1]);
  }
}

int Shortest_Path(const vector<vector<int> >& Grid, point start, point dest){
  // O(N^4),to make sure if a team does it, it won't be TLE
  vector<vector<int> > Distances(Grid.size());
  vector<vector<bool> > Visited(Grid.size());
  for(int i = 0; i < Grid.size(); i++){
    Distances[i].resize(Grid[i].size(), Grid.size()*Grid[0].size()*5);
    Visited[i].resize(Grid[i].size(), false);
  }
  Distances[start.r][start.c] = 0;
  Visited[start.r][start.c] = true;
  Relax(Grid, Distances, Visited, start);
  while(!Visited[dest.r][dest.c]){
    int cheapest = Grid.size()* Grid[0].size() * 5;
    point best;
    for(int i = 0; i < Visited.size(); i++){
      for(int j = 0; j < Visited[i].size(); j++){
	if(!Visited[i][j] && Distances[i][j] < cheapest){
	  best.r = i;
	  best.c = j;
	  cheapest = Distances[i][j];
	}
      }
    }
    //   cout << "best = (" << best.r <<"," << best.c <<")" << endl;
    // char c;
    // cin >> c;
    Visited[best.r][best.c] = true;
    Relax(Grid, Distances, Visited, best);
  }
  return Distances[dest.r][dest.c];
}

void Setup(vector<vector<int> > &Grid, int price){
  int num;
  cin >> num;
  for(int i = 0; i < num; i++){
    int r, c;
    cin >> r >> c;
    Grid[r][c] = price;
  }
}

int main(){
  int cur_case = 1;
  int r,c;
  cin >> r >> c;
  while(r > 0){
    vector<point> cities(2);
    vector<vector<int> > Grid(r);
    for(int i = 0; i < r; i++){
      Grid[i].resize(c, 1);
    }
    for(int i = 0; i < 2; i++){
      cin >> cities[i].r;
      cin >> cities[i].c;
    }

    Setup(Grid, 2); // read and process forests
    Setup(Grid, 3); // read and process mountains
    Setup(Grid, 0); // read and process track
    int cost = Shortest_Path(Grid, cities[0], cities[1]);
    cout << "Case " << cur_case << ": $" << cost << endl;
    cur_case++;
    cin >> r >> c;
  }
  return 0;
}

    
    
